"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const transform = {
    trimStart: (s) => s.trimStart(),
    trimEnd: (s) => s.trimEnd(),
    trimLeft: (s) => s.trimStart(),
    trimRight: (s) => s.trimEnd(),
    trim: (s) => s.trim(),
    toLowerCase: (s) => s.toLowerCase(),
    toUpperCase: (s) => s.toUpperCase(),
    toEnumCase: (s, cfg) => (cfg === null || cfg === void 0 ? void 0 : cfg.hash[configKey(s)]) || s,
};
function getDef() {
    return {
        keyword: "transform",
        type: "string",
        schemaType: "array",
        errors: false,
        modifying: true,
        valid: true,
        compile(schema, parentSchema) {
            let cfg;
            if (schema.includes("toEnumCase"))
                cfg = getEnumCaseCfg(parentSchema);
            return function (data, dataCxt) {
                // skip if top level value
                if (!dataCxt)
                    return true;
                const { parentData, parentDataProperty: key } = dataCxt;
                if (!parentData)
                    return true; // TODO fix in ajv: either dataCxt type should be changed or undefined context should be passed when parentData is undefined
                // apply transforms in order provided
                for (const t of schema)
                    data = transform[t](data, cfg);
                parentData[key] = data;
                return true;
            };
        },
        metaSchema: {
            type: "array",
            items: { type: "string", enum: Object.keys(transform) },
        },
    };
}
exports.default = getDef;
function getEnumCaseCfg(parentSchema) {
    // build hash table to enum values
    const cfg = { hash: {} };
    // requires `enum` in the same schema as transform
    if (!parentSchema.enum) {
        throw new Error('Missing enum. To use `transform:["toEnumCase"]`, `enum:[...]` is required.');
    }
    for (const v of parentSchema.enum) {
        if (typeof v !== "string")
            continue;
        const k = configKey(v);
        // requires all `enum` values have unique keys
        if (cfg.hash[k]) {
            throw new Error('Invalid enum uniqueness. To use `transform:["toEnumCase"]`, all values must be unique when case insensitive.');
        }
        cfg.hash[k] = v;
    }
    return cfg;
}
function configKey(s) {
    return s.toLowerCase();
}
module.exports = getDef;
//# sourceMappingURL=transform.js.map